package generated.copycardMehrereTerminals.terminal.depositMachine;

import swt.util.ByteArray;
import java.nio.charset.Charset;

/**
 * Class for T(L)V encoding of messages and other classes
 * 
 */
public class Coding {

	/**
	 * offset into encodeDestination for writing/reading
	 */
	private short encoding_length;

	/**
	 * array used for encoding/decoding
	 */
	private byte[] encodeDestination;

	/**
	 * Singleton instance
	 */
	private static Coding the_instance;

	private static Charset charset = Charset.forName("UTF-8");

	/**
	 * Singleton accessor
	 * 
	 * @return Singleton instance
	 */

	public static Coding getInstance() {
		if (the_instance == null)
			the_instance = new Coding();
		return the_instance;
	}
	/**
	 * return offset into encodeDestination
	 * 
	 * @return value of encoding_length
	 */
	public short getEncodingLength() {
		return encoding_length;
	}

	/**
	 * initializes encoding. This method must be called before encoding an object.
	 * The offset (encoding_length) is reset to 0 and encodeDestination is set to dest.
	 * 
	 * @param dest The (empty) destination array for encoding
	 */
	public void encodeInit(byte[] dest) {
		encodeDestination = dest;
		encoding_length = 0;
	}
	/**
	 * initializes decoding. The offset (encoding_length) is reset to 0.
	 */
	public void decodeInit() {
		encoding_length = 0;
	}

	/**
	 * initializes encoding and encodes an object.
	 * 
	 * @param o The object to encode
	 * @param dest destination array for the encoding
	 * @return length of the encoded object
	 */

	/**
	 * decodes an object beginning at given offset
	 * 
	 * @param in array containing encoded object
	 * @param offset where the encoded object begins
	 * @return the decoded object
	 */

	private byte decodeByte(byte[] in) throws TerminalException {
		if (in[encoding_length++] != Code.BYTE) {

			stop();
		}

		byte res = in[encoding_length++];
		return res;
	}

	private boolean decodeBoolean(byte[] in) throws TerminalException {
		if (in[encoding_length++] != Code.BOOLEAN) {
			stop();

		}
		boolean res = (in[encoding_length++] == 1 ? true : false);
		return res;
	}

	public Codeable decode(byte[] in, short offset) throws Exception {
		encoding_length = offset;
		return decode(in);
	}
	public byte[] stringToByte(String s) {
		byte[] b = s.getBytes(charset);
		byte[] ret = new byte[LengthConstants.LENGTHOFSTRING];
		int len = (b.length < LengthConstants.LENGTHOFSTRING)
				? b.length
				: LengthConstants.LENGTHOFSTRING;
		ByteArray.copy(b, 0, ret, 0, len);
		return ret;
	}

	public boolean isValidCardString(String s) {
		return (s.getBytes(charset).length <= LengthConstants.LENGTHOFSTRING);
	}

	public void encodeString(String i) {
		byte[] o = stringToByte(i);
		encodeDestination[encoding_length++] = Code.BYTEARRAY;
		ByteArray.setShort(encodeDestination, encoding_length,
				(short) (o.length));
		encoding_length += 2;
		ByteArray.copy(o, 0, encodeDestination, encoding_length, o.length);
		encoding_length += o.length;
	}

	private void compSizeString(String i) {
		byte[] o = stringToByte(i);
		encoding_length += 3;
		encoding_length += o.length;
	}

	public short encode(Codeable o, byte[] dest) throws Exception {
		encodeInit(dest);

		encode(o);
		return computeSize(o);

	}

	/**
	 * computes the encoding length of an object without actually encoding it.
	 * 
	 * @param o The object of which the encoding length should be computed
	 * @return encoding length
	 */

	public short computeSize(Codeable o) {
		encoding_length = 0;
		try {
			compSize(o);
			return encoding_length;
		} catch (Exception e) {
			return -1;
		}
	}

	public void encodeByte(byte o) {
		encodeDestination[encoding_length++] = Code.BYTE;
		encodeDestination[encoding_length++] = o;
	}

	private void encodeBoolean(boolean o) {
		encodeDestination[encoding_length++] = Code.BOOLEAN;
		encodeDestination[encoding_length++] = (byte) (o ? 1 : 0);
	}

	private void compSizeByte(byte o) {
		encoding_length += 2;
	}

	public void encodeInt(int i) {
		short o = (short) i;
		if (i != (int) o)
			System.err.println("int too big for cast into short.");
		encodeDestination[encoding_length++] = Code.SHORT;
		ByteArray.setShort(encodeDestination, encoding_length, o);
		encoding_length += 2;
	}

	private void compSizeInt(int i) {
		short o = (short) i;
		encoding_length += 3;
	}
	public int decodeInt(byte[] in) {
		short res = ByteArray.getShort(in, ++encoding_length);
		encoding_length += 2;
		return (int) res;
	}

	public byte[] decodeByteArray(byte[] in) {
		short len = ByteArray.getShort(in, ++encoding_length);
		byte[] dest = new byte[len];
		encoding_length += 2;
		ByteArray.copy(in, encoding_length, dest, 0, len);
		encoding_length += len;
		return dest;
	}

	public String byteToString(byte[] b) {
		int len = b.length;
		while (len > 0 && b[len - 1] == 0)
			len--;
		return new String(ByteArray.subarray(b, 0, len), charset);
	}

	public String decodeString(byte[] in) {
		short len = ByteArray.getShort(in, ++encoding_length);
		byte[] dest = new byte[len];
		encoding_length += 2;
		ByteArray.copy(in, encoding_length, dest, 0, len);
		encoding_length += len;
		String str = byteToString(dest);
		return str;
	}

	private void compSizeBoolean(boolean o) {
		encoding_length += 2;
	}

	public byte[] encodeSingleString(String s) {
		byte[] o = stringToByte(s);
		byte[] dest = new byte[3 + o.length];
		int i = 0;
		dest[i++] = Code.BYTEARRAY;
		ByteArray.setShort(dest, i, (short) (o.length));
		i += 2;
		ByteArray.copy(o, 0, dest, i, o.length);
		i += o.length;
		return dest;
	}

	public void encodeByteArray(byte[] o) {

		//byte[] o = stringToByte(i);

		encodeDestination[encoding_length++] = Code.BYTEARRAY;

		ByteArray.setShort(encodeDestination, encoding_length,
				(short) (o.length));
		encoding_length += 2;

		ByteArray.copy(o, 0, encodeDestination, encoding_length, o.length);

		encoding_length += o.length;
	}

	private void compSizeByteArray(byte[] o) {
		encoding_length += 3;
		encoding_length += o.length;
	}

	/**
	 * encodes an object without initializing destination and offset.
	 * 
	 * @param c object to encode
	 */

	public void encode(Codeable c) throws Exception {

		switch (c.getCode()) {
			case Code.LOAD :
				encodeLoad((Load) c);
				break;
			case Code.REQUESTBALANCE :
				encodeRequestBalance((RequestBalance) c);
				break;
			case Code.PAY :
				encodePay((Pay) c);
				break;
			case Code.RESREQUESTBALANCE :
				encodeResRequestBalance((ResRequestBalance) c);
				break;
			case Code.RESPAY :
				encodeResPay((ResPay) c);
				break;
			case Code.RESAUTHENTICATE :
				encodeResAuthenticate((ResAuthenticate) c);
				break;
			case Code.UREQUESTBALANCE :
				encodeURequestBalance((URequestBalance) c);
				break;
			case Code.UREQUESTCOPIES :
				encodeURequestCopies((URequestCopies) c);
				break;
			case Code.UINSERTMONEY :
				encodeUInsertMoney((UInsertMoney) c);
				break;
			case Code.AUTHENTICATE :
				encodeAuthenticate((Authenticate) c);
				break;
			case Code.UISSUECOPIES :
				encodeUIssueCopies((UIssueCopies) c);
				break;
			case Code.USHOWBALANCE :
				encodeUShowBalance((UShowBalance) c);
				break;
			case Code.RESLOAD :
				encodeResLoad((ResLoad) c);
				break;
			case Code.URESLOAD :
				encodeUResLoad((UResLoad) c);
				break;
			case Code.HASHEDDATA :
				encodeHashedData((HashedData) c);
				break;
			case Code.AUTHDATA :
				encodeAuthData((AuthData) c);
				break;
			case Code.NONCE :
				encodeNonce((Nonce) c);
				break;
			case Code.SECRET :
				encodeSecret((Secret) c);
				break;
			// never reached
			default :

				stop();

		}
	}

	public Codeable decode(byte[] in) throws Exception {
		switch (in[encoding_length]) {
			case Code.LOAD :
				return decodeLoad(in);
			case Code.REQUESTBALANCE :
				return decodeRequestBalance(in);
			case Code.PAY :
				return decodePay(in);
			case Code.RESREQUESTBALANCE :
				return decodeResRequestBalance(in);
			case Code.RESPAY :
				return decodeResPay(in);
			case Code.RESAUTHENTICATE :
				return decodeResAuthenticate(in);
			case Code.UREQUESTBALANCE :
				return decodeURequestBalance(in);
			case Code.UREQUESTCOPIES :
				return decodeURequestCopies(in);
			case Code.UINSERTMONEY :
				return decodeUInsertMoney(in);
			case Code.AUTHENTICATE :
				return decodeAuthenticate(in);
			case Code.UISSUECOPIES :
				return decodeUIssueCopies(in);
			case Code.USHOWBALANCE :
				return decodeUShowBalance(in);
			case Code.RESLOAD :
				return decodeResLoad(in);
			case Code.URESLOAD :
				return decodeUResLoad(in);
			case Code.HASHEDDATA :
				return decodeHashedData(in);
			case Code.AUTHDATA :
				return decodeAuthData(in);
			case Code.NONCE :
				return decodeNonce(in);
			case Code.SECRET :
				return decodeSecret(in);
				// never reached
			default :
				stop();
		}
		return null;
	}

	public void compSize(Codeable c) throws Exception {
		switch (c.getCode()) {
			case Code.LOAD :

				compSizeLoad((Load) c);

				break;
			case Code.REQUESTBALANCE :

				compSizeRequestBalance((RequestBalance) c);

				break;
			case Code.PAY :

				compSizePay((Pay) c);

				break;
			case Code.RESREQUESTBALANCE :

				compSizeResRequestBalance((ResRequestBalance) c);

				break;
			case Code.RESPAY :

				compSizeResPay((ResPay) c);

				break;
			case Code.RESAUTHENTICATE :

				compSizeResAuthenticate((ResAuthenticate) c);

				break;
			case Code.UREQUESTBALANCE :

				compSizeURequestBalance((URequestBalance) c);

				break;
			case Code.UREQUESTCOPIES :

				compSizeURequestCopies((URequestCopies) c);

				break;
			case Code.UINSERTMONEY :

				compSizeUInsertMoney((UInsertMoney) c);

				break;
			case Code.AUTHENTICATE :

				compSizeAuthenticate((Authenticate) c);

				break;
			case Code.UISSUECOPIES :

				compSizeUIssueCopies((UIssueCopies) c);

				break;
			case Code.USHOWBALANCE :

				compSizeUShowBalance((UShowBalance) c);

				break;
			case Code.RESLOAD :

				compSizeResLoad((ResLoad) c);

				break;
			case Code.URESLOAD :

				compSizeUResLoad((UResLoad) c);

				break;
			case Code.HASHEDDATA :

				compSizeHashedData((HashedData) c);

				break;
			case Code.AUTHDATA :

				compSizeAuthData((AuthData) c);

				break;
			case Code.NONCE :

				compSizeNonce((Nonce) c);

				break;
			case Code.SECRET :

				compSizeSecret((Secret) c);

				break;
			// never reached
			default :

				stop();

		}
	}

	public void encodeLoad(Load c) throws Exception {
		encodeDestination[encoding_length++] = Code.LOAD;

		encodeInt((int) c.getAmount());

		encode(c.getAuthterminal());
	}

	public Load decodeLoad(byte[] in) throws Exception {
		Load c = new Load();
		encoding_length++;
		c.setAmount(decodeInt(in));
		c.setAuthterminal(((HashedData) decode(in)));
		return c;
	}

	private void compSizeLoad(Load c) throws Exception {
		encoding_length++;

		compSizeInt(c.getAmount());
		compSize(c.getAuthterminal());
	}

	public void encodeRequestBalance(RequestBalance c) throws Exception {
		encodeDestination[encoding_length++] = Code.REQUESTBALANCE;
	}

	public RequestBalance decodeRequestBalance(byte[] in) throws Exception {
		RequestBalance c = new RequestBalance();
		encoding_length++;
		return c;
	}

	private void compSizeRequestBalance(RequestBalance c) throws Exception {
		encoding_length++;
	}

	public void encodePay(Pay c) throws Exception {
		encodeDestination[encoding_length++] = Code.PAY;

		encodeInt((int) c.getAmount());

		encode(c.getTerminalchallenge());
	}

	public Pay decodePay(byte[] in) throws Exception {
		Pay c = new Pay();
		encoding_length++;
		c.setAmount(decodeInt(in));
		c.setTerminalchallenge(((Nonce) decode(in)));
		return c;
	}

	private void compSizePay(Pay c) throws Exception {
		encoding_length++;

		compSizeInt(c.getAmount());
		compSize(c.getTerminalchallenge());
	}

	public void encodeResRequestBalance(ResRequestBalance c) throws Exception {
		encodeDestination[encoding_length++] = Code.RESREQUESTBALANCE;

		encodeInt((int) c.getBalance());
	}

	public ResRequestBalance decodeResRequestBalance(byte[] in)
			throws Exception {
		ResRequestBalance c = new ResRequestBalance();
		encoding_length++;
		c.setBalance(decodeInt(in));
		return c;
	}

	private void compSizeResRequestBalance(ResRequestBalance c)
			throws Exception {
		encoding_length++;

		compSizeInt(c.getBalance());
	}

	public void encodeResPay(ResPay c) throws Exception {
		encodeDestination[encoding_length++] = Code.RESPAY;

		encode(c.getAuthcard());
	}

	public ResPay decodeResPay(byte[] in) throws Exception {
		ResPay c = new ResPay();
		encoding_length++;
		c.setAuthcard(((HashedData) decode(in)));
		return c;
	}

	private void compSizeResPay(ResPay c) throws Exception {
		encoding_length++;
		compSize(c.getAuthcard());
	}

	public void encodeResAuthenticate(ResAuthenticate c) throws Exception {
		encodeDestination[encoding_length++] = Code.RESAUTHENTICATE;

		encode(c.getCardletchallenge());
	}

	public ResAuthenticate decodeResAuthenticate(byte[] in) throws Exception {
		ResAuthenticate c = new ResAuthenticate();
		encoding_length++;
		c.setCardletchallenge(((Nonce) decode(in)));
		return c;
	}

	private void compSizeResAuthenticate(ResAuthenticate c) throws Exception {
		encoding_length++;
		compSize(c.getCardletchallenge());
	}

	public void encodeURequestBalance(URequestBalance c) throws Exception {
		encodeDestination[encoding_length++] = Code.UREQUESTBALANCE;
	}

	public URequestBalance decodeURequestBalance(byte[] in) throws Exception {
		URequestBalance c = new URequestBalance();
		encoding_length++;
		return c;
	}

	private void compSizeURequestBalance(URequestBalance c) throws Exception {
		encoding_length++;
	}

	public void encodeURequestCopies(URequestCopies c) throws Exception {
		encodeDestination[encoding_length++] = Code.UREQUESTCOPIES;

		encodeInt((int) c.getValue());
	}

	public URequestCopies decodeURequestCopies(byte[] in) throws Exception {
		URequestCopies c = new URequestCopies();
		encoding_length++;
		c.setValue(decodeInt(in));
		return c;
	}

	private void compSizeURequestCopies(URequestCopies c) throws Exception {
		encoding_length++;

		compSizeInt(c.getValue());
	}

	public void encodeUInsertMoney(UInsertMoney c) throws Exception {
		encodeDestination[encoding_length++] = Code.UINSERTMONEY;

		encodeInt((int) c.getValue());
	}

	public UInsertMoney decodeUInsertMoney(byte[] in) throws Exception {
		UInsertMoney c = new UInsertMoney();
		encoding_length++;
		c.setValue(decodeInt(in));
		return c;
	}

	private void compSizeUInsertMoney(UInsertMoney c) throws Exception {
		encoding_length++;

		compSizeInt(c.getValue());
	}

	public void encodeAuthenticate(Authenticate c) throws Exception {
		encodeDestination[encoding_length++] = Code.AUTHENTICATE;
	}

	public Authenticate decodeAuthenticate(byte[] in) throws Exception {
		Authenticate c = new Authenticate();
		encoding_length++;
		return c;
	}

	private void compSizeAuthenticate(Authenticate c) throws Exception {
		encoding_length++;
	}

	public void encodeUIssueCopies(UIssueCopies c) throws Exception {
		encodeDestination[encoding_length++] = Code.UISSUECOPIES;

		encodeInt((int) c.getValue());
	}

	public UIssueCopies decodeUIssueCopies(byte[] in) throws Exception {
		UIssueCopies c = new UIssueCopies();
		encoding_length++;
		c.setValue(decodeInt(in));
		return c;
	}

	private void compSizeUIssueCopies(UIssueCopies c) throws Exception {
		encoding_length++;

		compSizeInt(c.getValue());
	}

	public void encodeUShowBalance(UShowBalance c) throws Exception {
		encodeDestination[encoding_length++] = Code.USHOWBALANCE;

		encodeInt((int) c.getValue());
	}

	public UShowBalance decodeUShowBalance(byte[] in) throws Exception {
		UShowBalance c = new UShowBalance();
		encoding_length++;
		c.setValue(decodeInt(in));
		return c;
	}

	private void compSizeUShowBalance(UShowBalance c) throws Exception {
		encoding_length++;

		compSizeInt(c.getValue());
	}

	public void encodeResLoad(ResLoad c) throws Exception {
		encodeDestination[encoding_length++] = Code.RESLOAD;

		encodeInt((int) c.getBalance());
	}

	public ResLoad decodeResLoad(byte[] in) throws Exception {
		ResLoad c = new ResLoad();
		encoding_length++;
		c.setBalance(decodeInt(in));
		return c;
	}

	private void compSizeResLoad(ResLoad c) throws Exception {
		encoding_length++;

		compSizeInt(c.getBalance());
	}

	public void encodeUResLoad(UResLoad c) throws Exception {
		encodeDestination[encoding_length++] = Code.URESLOAD;

		encodeInt((int) c.getBalance());

		encodeInt((int) c.getAmountToLoad());
	}

	public UResLoad decodeUResLoad(byte[] in) throws Exception {
		UResLoad c = new UResLoad();
		encoding_length++;
		c.setBalance(decodeInt(in));
		c.setAmountToLoad(decodeInt(in));
		return c;
	}

	private void compSizeUResLoad(UResLoad c) throws Exception {
		encoding_length++;

		compSizeInt(c.getBalance());

		compSizeInt(c.getAmountToLoad());
	}

	public void encodeHashedData(HashedData c) throws Exception {
		encodeDestination[encoding_length++] = Code.HASHEDDATA;

		encodeByteArray((byte[]) c.getHashed());
	}

	public HashedData decodeHashedData(byte[] in) throws Exception {
		HashedData c = new HashedData();
		encoding_length++;
		c.setHashed(decodeByteArray(in));
		return c;
	}

	private void compSizeHashedData(HashedData c) throws Exception {
		encoding_length++;

		compSizeByteArray(c.getHashed());
	}

	public void encodeAuthData(AuthData c) throws Exception {
		encodeDestination[encoding_length++] = Code.AUTHDATA;

		encodeInt((int) c.getInstruction());

		encode(c.getPassphrase());

		encode(c.getChallenge());

		encodeInt((int) c.getAmount());
	}

	public AuthData decodeAuthData(byte[] in) throws Exception {
		AuthData c = new AuthData();
		encoding_length++;
		c.setInstruction(decodeInt(in));
		c.setPassphrase(((Secret) decode(in)));
		c.setChallenge(((Nonce) decode(in)));
		c.setAmount(decodeInt(in));
		return c;
	}

	private void compSizeAuthData(AuthData c) throws Exception {
		encoding_length++;

		compSizeInt(c.getInstruction());
		compSize(c.getPassphrase());
		compSize(c.getChallenge());

		compSizeInt(c.getAmount());
	}

	public void encodeNonce(Nonce c) throws Exception {
		encodeDestination[encoding_length++] = Code.NONCE;

		encodeByteArray((byte[]) c.getNonce());
	}

	public Nonce decodeNonce(byte[] in) throws Exception {
		Nonce c = new Nonce();
		encoding_length++;
		c.setNonce(decodeByteArray(in));
		return c;
	}

	private void compSizeNonce(Nonce c) throws Exception {
		encoding_length++;

		compSizeByteArray(c.getNonce());
	}

	public void encodeSecret(Secret c) throws Exception {
		encodeDestination[encoding_length++] = Code.SECRET;

		encodeByteArray((byte[]) c.getSecret());
	}

	public Secret decodeSecret(byte[] in) throws Exception {
		Secret c = new Secret();
		encoding_length++;
		c.setSecret(decodeByteArray(in));
		return c;
	}

	private void compSizeSecret(Secret c) throws Exception {
		encoding_length++;

		compSizeByteArray(c.getSecret());
	}

	private void stop() throws TerminalException {
		throw new TerminalException("Coding");
	}
}
